package org.purpurmc.purpur.task;

import io.canvasmc.canvas.ThreadedBukkitServer;
import io.canvasmc.canvas.command.ThreadedServerHealthDump;
import io.canvasmc.canvas.region.ChunkRegion;
import io.canvasmc.canvas.region.ServerRegions;
import io.papermc.paper.threadedregions.ThreadedRegionizer;
import net.kyori.adventure.bossbar.BossBar;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.TextColor;
import net.kyori.adventure.text.minimessage.MiniMessage;
import net.kyori.adventure.text.minimessage.tag.resolver.Placeholder;
import net.kyori.adventure.util.HSVLike;
import net.minecraft.server.MinecraftServer;
import net.minecraft.server.level.ServerLevel;
import net.minecraft.server.level.ServerPlayer;
import org.bukkit.craftbukkit.entity.CraftPlayer;
import org.purpurmc.purpur.PurpurConfig;
import org.bukkit.entity.Player;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

public class TPSBarTask extends BossBarTask {
    private static TPSBarTask instance;
    // Canvas start - Replaced doubles (tps and mspt) with maps to get data of player current world
    private final Map<UUID, Double> tps = new HashMap<>(0) {
        @Override
        public Double get(final Object key) {
            return this.computeIfAbsent((UUID) key, (_) -> 0.0D);
        }
    };
    private final Map<UUID, Double> mspt = new HashMap<>(0) {
        @Override
        public Double get(final Object key) {
            return this.computeIfAbsent((UUID) key, (_) -> 0.0D);
        }
    };
    // Canvas end
    private int tick = 0;
    private final RegionBarTask regionBarTask = new RegionBarTask(); // Canvas - Threaded Regions

    public static TPSBarTask instance() {
        if (instance == null) {
            instance = new TPSBarTask();
        }
        return instance;
    }

    @Override
    BossBar createBossBar() {
        return BossBar.bossBar(Component.text(""), 0.0F, instance().getBossBarColor(), PurpurConfig.commandTPSBarProgressOverlay);
    }

    @Override
    void updateBossBar(BossBar bossbar, Player player) {
        // Canvas start - get player's current world UUID and pass it to some of the methods for showing TPS and MSPT of current world
        UUID playerWorldUID = player.getWorld().getUID();

        bossbar.progress(getBossBarProgress(playerWorldUID));
        bossbar.color(getBossBarColor());
        bossbar.name(MiniMessage.miniMessage().deserialize(PurpurConfig.commandTPSBarTitle,
            Placeholder.component("tps", ThreadedServerHealthDump.getTPSColor(((CraftPlayer) player).getHandle().serverLevel().getTps5s().getAverage())),
            Placeholder.component("mspt", ThreadedServerHealthDump.getMSPTColor(((CraftPlayer) player).getHandle().serverLevel().getTickTimes5s().getAverage())),
            Placeholder.component("ping", getPingColor(player.getPing()))
        ));
        // Canvas end
    }

    @Override
    public void run() {
        if (++tick < PurpurConfig.commandTPSBarTickInterval || tick < MinecraftServer.getServer().tickRateManager().tickrate()) { // Canvas - don't run task more often than once a second
            return;
        }
        tick = 0;

        // Canvas start - Per level TPS and MSPT
        for (ServerLevel level : MinecraftServer.getThreadedServer().getThreadedWorlds()) {
            tps.put(level.uuid, Math.max(Math.min(level.tps5s.getAverage(), 20.0D), 0.0D));
            mspt.put(level.uuid, level.tickTimes5s.getAverage());
        }
        // Canvas end

        super.run();
    }

    private float getBossBarProgress(UUID playerWorldUID) {  // Canvas - Add UUID as an argument
        if (PurpurConfig.commandTPSBarProgressFillMode == FillMode.MSPT) {
            return Math.max(Math.min(mspt.get(playerWorldUID).floatValue() / ThreadedBukkitServer.getInstance().getScheduler().getTimeBetweenTicks(), 1.0F), 0.0F); // Canvas - get MSPT from map
        } else {
            return Math.max(Math.min(tps.get(playerWorldUID).floatValue() / ThreadedBukkitServer.getInstance().getScheduler().getTickRate(), 1.0F), 0.0F); // Canvas - get TPS from map
        }
    }

    private BossBar.Color getBossBarColor() {
        return BossBar.Color.GREEN; // Canvas - rwerite TPS bars
    }

    // Canvas start - rewrite TPS bars
    private static Component getPingColor(int ping) {
        final double clamped = Math.min(Math.max(ping, 0), 300);
        final double hue = 130.0 - (130.0 * (clamped / 300.0));
        final float saturation = 0.85f;
        final float value = 0.80f;

        return Component.text(String.valueOf(ping))
            .color(TextColor.color(HSVLike.hsvLike((float)(hue / 360.0), saturation, value)));
    }
    // Canvas end

    public enum FillMode {
        TPS, MSPT, PING
    }
    // Canvas start - Threaded Regions

    public static class RegionBarTask extends BossBarTask {
        private static org.purpurmc.purpur.task.TPSBarTask.RegionBarTask instance;
        private int tick = 0;

        public static org.purpurmc.purpur.task.TPSBarTask.RegionBarTask instance() {
            if (instance == null) {
                instance = new org.purpurmc.purpur.task.TPSBarTask.RegionBarTask();
            }
            return instance;
        }

        @Override
        BossBar createBossBar() {
            return BossBar.bossBar(Component.text(""), 0.0F, BossBar.Color.BLUE, PurpurConfig.commandTPSBarProgressOverlay);
        }

        @Override
        void updateBossBar(BossBar bossbar, Player player) {
            ServerPlayer serverPlayer = ((CraftPlayer) player).getHandle();
            ThreadedRegionizer.ThreadedRegion<ServerRegions.TickRegionData, ServerRegions.TickRegionSectionData> chunkRegion = serverPlayer.serverLevel().regioniser.getRegionAtUnsynchronised(serverPlayer.chunkPosition().x, serverPlayer.chunkPosition().z);
            if (chunkRegion == null) {
                return;
            }

            bossbar.progress(getBossBarProgress(chunkRegion.getData().tickHandle));
            bossbar.color(BossBar.Color.BLUE);
            bossbar.name(MiniMessage.miniMessage().deserialize("<gray>Region TPS<yellow>:</yellow> <tps> Region MSPT<yellow>:</yellow> <mspt>",
                Placeholder.component("tps", ThreadedServerHealthDump.getTPSColor(chunkRegion.getData().tickHandle.getTps5s().getAverage())),
                Placeholder.component("mspt", ThreadedServerHealthDump.getMSPTColor(chunkRegion.getData().tickHandle.getTickTimes5s().getAverage()))
            ));
        }

        @Override
        public void run() {
            if (++tick < PurpurConfig.commandTPSBarTickInterval || tick < MinecraftServer.getServer().tickRateManager().tickrate()) {
                return;
            }
            tick = 0;
            super.run();
        }

        private float getBossBarProgress(ChunkRegion region) {
            if (PurpurConfig.commandTPSBarProgressFillMode == org.purpurmc.purpur.task.TPSBarTask.FillMode.MSPT) {
                return Math.max(Math.min(((float) region.tickTimes5s.getAverage()) / ThreadedBukkitServer.getInstance().getScheduler().getTimeBetweenTicks(), 1.0F), 0.0F);
            } else {
                return Math.max(Math.min(((float) region.tps5s.getAverage()) / ThreadedBukkitServer.getInstance().getScheduler().getTickRate(), 1.0F), 0.0F);
            }
        }
    }
    // Canvas end
}
